from aistore.sdk.api import Client

# If there is a version module generated by the setup script, import the version info into this module
# Otherwise ignore so the sdk works locally without running setup
try:
    from aistore.version import __version__
except ImportError:
    pass
